//DevMpf.cc
 
/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
        Current Author: Marty Kraimer
        Original Author: Jim Kowalkowski (for Hideos)
        Date: 01JUL98
*/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>

extern "C" {
#include "dbDefs.h"
#include "link.h"
#include "dbAccess.h"
#include "recSup.h"
#include  "dbScan.h"
}
 
#include "Message.h"
#include "ConnectMessage.h"

#include "DevMpf.h"

int DevMpfDebug = 0;

static void callDeviceSupport(CALLBACK* cback);

DevMpf::DevMpf(dbCommon* pr, link* pl,bool iointvalid)
: status(0), precord(pr), plink(pl), userParm(0), pMessageClient(0),
  ioscanpvt(0), iointValid(iointvalid),
  connectState(connectNo), replyMessage(0)
{
    char name[40];
    int i;
    if(plink->type!=VME_IO) {
	epicsPrintf("%s INP is not a vme link\n",precord->name);
	precord->pact = true;
	return(this);
    }
    pMessageClient = new MessageClient(DevMpf::messageClientCallback,
	(void *)precord);
    vmeio *pvmeio=(vmeio*)&(plink->value);
    // first field of parm is always the server name
    for(i=0;
       pvmeio->parm[i] && pvmeio->parm[i]!=',' && pvmeio->parm[i]!=' ';
       i++) name[i]=pvmeio->parm[i];
    if(pvmeio->parm[i]) userParm = &pvmeio->parm[i+1];
    name[i]='\0';
    callbackSetCallback(callDeviceSupport,&cb);	
    callbackSetPriority(precord->prio,&cb);
    callbackSetUser(precord,&cb);
    scanIoInit(&ioscanpvt);
    precord->dpvt=this;
    status = pMessageClient->bind(name,pvmeio->card);
    if(status) {
	epicsPrintf("MPF::bind error. server %s Record %s\n",
	    name,precord->name);
        precord->pact = true;
        return(this);
    }
    return(this);
}

long DevMpf::linconv(void* v,int pass)
{
    dbCommon* precord=(dbCommon*)v;
    DevMpf* pdevMpf=(DevMpf*)precord->dpvt;
    if(pdevMpf==0) return(0);
    pdevMpf->convert(precord,pass);
    return(0);
}

void DevMpf::connectIO(dbCommon*pcommon,Message *message)
{
    DevMpf *pdevMpf = (DevMpf *)pcommon->dpvt;
    delete message;
    if(pdevMpf->connectState==connectYes) return;
    callbackRequest(&pdevMpf->cb);
}

long DevMpf::convert(dbCommon*,int) { return(0); }

int DevMpf::send(Message *pmessage)
{
    int status = pMessageClient->send(pmessage);
    if(status!=0) {
	delete pmessage;
	recGblSetSevr(precord,READ_ALARM,INVALID_ALARM);
	return(MPF_NoConvert);
    }
    return(0);
}

int DevMpf::sendReply(Message *pmessage)
{
    int status = send(pmessage);
    if(status==0) precord->pact = TRUE;
    return(status);
}

long DevMpf::ioint(int , dbCommon* pcommon,IOSCANPVT* iopvt)
{
    DevMpf* pdevMpf=(DevMpf*)pcommon->dpvt;
    if(!pdevMpf->iointValid) return(0);
    if(pdevMpf==0) return(0);
    *iopvt=pdevMpf->ioscanpvt;
    return(0);
}

long DevMpf::read_write(void* v)
{
    dbCommon* pcommon = (dbCommon*)v;
    DevMpf* pdevMpf=(DevMpf*)pcommon->dpvt;
    if(pdevMpf==0 || pdevMpf->connectState!=connectYes) {
	if(pdevMpf->connectState==connectQueueFull)
	    pdevMpf->connectState = connectYes;
	recGblSetSevr(pcommon,READ_ALARM,INVALID_ALARM);
	if(pdevMpf->replyMessage) {
	    delete pdevMpf->replyMessage;
	    pdevMpf->replyMessage = 0;
	}
	return(MPF_NoConvert);
    }
    if(pcommon->pact==TRUE && pdevMpf->replyMessage==0) {
	if(DevMpfDebug) epicsPrintf("%s PACT true and no reply message\n");
	recGblSetSevr(pcommon,READ_ALARM,INVALID_ALARM);
	return(MPF_NoConvert);
    }
    long rc = 0;
    if(pdevMpf->replyMessage!=0) {
	rc=pdevMpf->completeIO(pcommon,pdevMpf->replyMessage);
	pdevMpf->replyMessage=0; // allow next message to be handled
    } else {
	rc=pdevMpf->startIO(pcommon);
    }	
    return rc;
}

typedef long (*processCall)(dbCommon *);

static void callDeviceSupport(CALLBACK* cback)
{
    dbCommon* pc;
    callbackGetUser(pc,cback);
    rset *prset=(struct rset *)(pc->rset);
    processCall pcall = (processCall)(prset->process);
    /* process the record */
    dbScanLock(pc);
    (*pcall)(pc);
    dbScanUnlock(pc);
}

void DevMpf::messageClientCallback(Message *message,void *clientPvt)
{
    dbCommon *pcommon = (dbCommon *)clientPvt;
    DevMpf *pdevMpf = (DevMpf *)pcommon->dpvt;

    if(message->getType()==messageTypeConnect) {
	ConnectMessage *pConnectMessage = (ConnectMessage *)message;
	pdevMpf->connectState = pConnectMessage->status;
	pdevMpf->connectIO(pcommon,message);
    } else {
        pdevMpf->replyMessage = message;
        callbackRequest(&pdevMpf->cb);
    }
}
