//GpibHideosRemote.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
    Autor:	Tom Coleman
    Date:	1995/Dec/10

*/
/*
This is the interface called by drvGpib.
It was originally written for hideos. To keep compatibility 
mpf uses the same interface definition with a different implementation.
*/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <vxWorks.h>
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
#include <vme.h>
#include <iv.h>
 
extern "C" {
#include "drvHiDEOSGpib.h"
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "stringinRecord.h"
#include "stringoutRecord.h"
#include "recSup.h"
}
 
#include "Message.h"
#include "Char8ArrayMessage.h"
#include "Int32Message.h"
#include "DevMpf.h"
#include "gpibServer.h"

extern "C" {
void *GpibHideosInit(int BoardId, char *TaskName);
int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
              unsigned long devAddr, unsigned long TimeOut);
int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
              unsigned long *Actual, unsigned long devAddr, 
              unsigned long TimeOut, long Eos);
int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
              char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
              unsigned long devAddr, unsigned long TimeOut, long Eos);
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
              unsigned long TimeOut);
};

struct pvtData {
    MessageClient *pMessageClient;
    connectStatus connectState;
    SEM_ID	sem;
    //following for each message sent to server
    bool active;
    gpibCommand cmd;
    Int32Message *int32Message;
    Char8ArrayMessage *stringMessage;
};

inline void initPvtData(pvtData *p,gpibCommand cmd)
{
  p->cmd=cmd; p->int32Message=0; p->stringMessage=0;
  //   printf ("GpibHideos initPvtData pvtData=%p cmd=%d\n",p,cmd);
}

static void myCallback(Message *message,void *clientPvt)
{
  pvtData *ppvtData = (pvtData *)clientPvt;
  
  //   printf("myCallback messageType %d\n",message->getType());
  switch(message->getType()) {
  case messageTypeConnect:
    {
      ConnectMessage *pConnectMessage = (ConnectMessage *)message;
      ppvtData->connectState = pConnectMessage->status;
      delete pConnectMessage;
    }
    semGive(ppvtData->sem);
    break;
  case messageTypeInt32:
    ppvtData->int32Message = (Int32Message *)message;
    semGive(ppvtData->sem);
    break;
  case messageTypeChar8Array:
    ppvtData->stringMessage = (Char8ArrayMessage *)message;
    semGive(ppvtData->sem);
    break;
  default:
    //   printf ("GpibHideos: Uknown Message Type!\n");
    break;
  }
}

void *GpibHideosInit(int location, char *server)
{
  //   printf("GpibHideosInit[Remote]\n");
    pvtData *ppvtData = new pvtData;
    ::memset(ppvtData,0,sizeof(pvtData));
    ppvtData->connectState = connectNo;
    ppvtData->sem  = semBCreate(SEM_Q_PRIORITY,SEM_EMPTY);
    MessageClient *pMessageClient = new MessageClient(myCallback,ppvtData);
    ppvtData->pMessageClient = pMessageClient;
    //   printf("GpibHideosInit[Remote]: bind %d %s\n",location, server);
    int status = pMessageClient->bind(server,location);
    semTake(ppvtData->sem,WAIT_FOREVER);
    //   printf("GpibHideosInit[Remote]: bind status %d\n",status);
    //   printf("GpibHideosInit[Remote]: connectState %d\n",ppvtData->connectState);
    if(ppvtData->connectState != connectYes) return(0);
    if(status) {
        printf("GpibHideosInit[Remote]: unable to bind to server %d %s\n", location, server);
	return(0);
    }
    return(ppvtData);
}

int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
        unsigned long devAddr, unsigned long TimeOut)
{
  //  printf("GpibHideosWrite[Remote]\n");
  //  printf("GpibHideosWrite[Remote] td %p Buf %p BufLen %ld devAddr %ld TimeOut %ld\n", td, Buf, BufLen, devAddr, TimeOut);
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState != connectYes) return(ERROR);
    initPvtData(ppvtData,gpibWrite);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Char8ArrayMessage *sendMessage = new Char8ArrayMessage;
    sendMessage->timeout = (int32)TimeOut;
    sendMessage->cmd = gpibWrite;
    sendMessage->address = (int32)devAddr;
    sendMessage->allocValue(BufLen);
    sendMessage->setSize(BufLen);
    ::memcpy(sendMessage->value,Buf,BufLen);
    int status = pMessageClient->send(sendMessage);
    //   printf("GpibHideosWrite[Remote] pMessageClient->send %d\n", status);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
//    printf("GpibHideosWrite[Remote] ppvtData->cmd %d\n", ppvtData->cmd);
//    printf("GpibHideosWrite[Remote] ppvtData->int32Message %p\n", ppvtData->int32Message);
//    printf("GpibHideosWrite[Remote] ppvtData->stringMessage %p\n", ppvtData->stringMessage);
    if(ppvtData->connectState != connectYes) return(ERROR);
    Char8ArrayMessage *receiveMessage = ppvtData->stringMessage;
//    printf("GpibHideosWrite[Remote] receiveMessage->status %d\n", receiveMessage->status);
    status = receiveMessage->status;
    delete receiveMessage;
//    printf("GpibHideosWrite[Remote] return(%d);\n", status);
    return(status);
}

int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
            unsigned long *Actual, unsigned long devAddr, 
            unsigned long TimeOut, long Eos)
{
  //   printf("GpibHideosRead[Remote]\n");
  //   printf("GpibHideosRead[Remote] td %p Buf %p BufLen %ld devAddr %ld TimeOut %ld Eos %ld\n", td, Buf, BufLen, devAddr, TimeOut, Eos);
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState != connectYes) return(ERROR);
    initPvtData(ppvtData,gpibRead);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Int32Message *sendMessage = new Int32Message;
    sendMessage->timeout = (int32)TimeOut;
    sendMessage->cmd = gpibRead;
    sendMessage->address = (int32)devAddr;
    sendMessage->extra = (int32)Eos;
    sendMessage->value = (int32)BufLen;
    int status = pMessageClient->send(sendMessage);
    //   printf("GpibHideosRead[Remote] pMessageClient->send %d\n", status);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    //   printf("GpibHideosRead[Remote] ppvtData->cmd %d\n", ppvtData->cmd);
    //   printf("GpibHideosRead[Remote] ppvtData->int32Message %p\n", ppvtData->int32Message);
    //   printf("GpibHideosRead[Remote] ppvtData->stringMessage %p\n", ppvtData->stringMessage);
    if(ppvtData->connectState != connectYes) return(ERROR);
    Char8ArrayMessage *receiveMessage = ppvtData->stringMessage;
    *Actual = min((unsigned long)receiveMessage->getSize(),BufLen);
    //   printf("GpibHideosRead[Remote] *Actual %ld\n", *Actual);
    ::memcpy(Buf,receiveMessage->value,*Actual);
    //   printf("GpibHideosRead[Remote] receiveMessage->status %d\n", receiveMessage->status);
    status = receiveMessage->status;
    delete receiveMessage;
    if(status) return(ERROR);
    //   printf("GpibHideosRead[Remote] return(0);\n");
    return(0);
}

int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
        char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
        unsigned long devAddr, unsigned long TimeOut, long Eos)
{
  //   printf("GpibHideosWriteRead[Remote]\n");
  //   printf("GpibHideosWriteRead[Remote] td %p WBuf %p WBufLen %ld RBuf %p RBufLen %ld devAddr %ld TimeOut %ld Eos %ld\n", td, WBuf, WBufLen, RBuf, RBufLen, devAddr, TimeOut, Eos);
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState != connectYes) return(ERROR);
    initPvtData(ppvtData,gpibWriteRead);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Char8ArrayMessage *sendMessage = new Char8ArrayMessage;
    sendMessage->timeout = (int32)TimeOut;
    sendMessage->cmd = gpibWriteRead;
    sendMessage->address = (int32)devAddr;
    sendMessage->extra = (int32)Eos;
    sendMessage->allocValue(WBufLen);
    sendMessage->setSize(WBufLen);
    ::memcpy(sendMessage->value,WBuf,WBufLen);
    int status = pMessageClient->send(sendMessage);
    //   printf("GpibHideosWriteRead[Remote] pMessageClient->send %d\n", status);
    semTake(ppvtData->sem,WAIT_FOREVER);
    //   printf("GpibHideosWriteRead[Remote] ppvtData->cmd %d\n", ppvtData->cmd);
    //   printf("GpibHideosWriteRead[Remote] ppvtData->int32Message %p\n", ppvtData->int32Message);
    //   printf("GpibHideosWriteRead[Remote] ppvtData->stringMessage %p\n", ppvtData->stringMessage);
    if(ppvtData->connectState != connectYes) return(ERROR);
    Char8ArrayMessage *receiveMessage = ppvtData->stringMessage;
    *RActual = min((unsigned long)receiveMessage->getSize(),RBufLen);
    //   printf("GpibHideosWriteRead[Remote] *RActual %ld\n", *RActual);
    ::memcpy(RBuf,receiveMessage->value,*RActual);
    //   printf("GpibHideosWriteRead[Remote] receiveMessage->status %d\n", receiveMessage->status);
    status = receiveMessage->status;
    delete receiveMessage;
    if(status) return(ERROR);
    //   printf("GpibHideosWriteRead[Remote] return(0);\n");
    return(0);
}

int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
                unsigned long TimeOut)
{
  //   printf("GpibHideosWriteCmd[Remote]\n");
  //   printf("GpibHideosWriteCmd[Remote] td %p Buf %p BufLen %ld TimeOut %ld\n", td, Buf, BufLen, TimeOut);
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState != connectYes) return(ERROR);
    initPvtData(ppvtData,gpibCmd);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Char8ArrayMessage *sendMessage = new Char8ArrayMessage;
    sendMessage->timeout = (int32)TimeOut; 
    sendMessage->cmd = gpibCmd;
    sendMessage->allocValue(BufLen);
    sendMessage->setSize(BufLen);
    ::memcpy(sendMessage->value,Buf,BufLen);
    int status = pMessageClient->send(sendMessage);
    //   printf("GpibHideosWriteCmd[Remote] pMessageClient->send %d\n", status);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    //   printf("GpibHideosWriteCmd[Remote] ppvtData->cmd %d\n", ppvtData->cmd);
    //   printf("GpibHideosWriteCmd[Remote] ppvtData->int32Message %p\n", ppvtData->int32Message);
    //   printf("GpibHideosWriteCmd[Remote] ppvtData->stringMessage %p\n", ppvtData->stringMessage);
    if(ppvtData->connectState != connectYes) return(ERROR);
    Char8ArrayMessage *receiveMessage = ppvtData->stringMessage;
    //   printf("GpibHideosWriteCmd[Remote] receiveMessage->status %d\n", receiveMessage->status);
    status = receiveMessage->status;
    delete receiveMessage;
    //   printf("GpibHideosWriteCmd[Remote] return(%d);\n", status);
    return(status);
}
