//Gpib.h
 
/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
        Author: Marty Kraimer
        Original Author: Jim Kowalkowski (for Hideos)
        Date: 22JUL98
*/
 
#ifndef GpibH
#define GpibH

#include <semLib.h>

enum gpibStatus {gpibSuccess,gpibTimeout,gpibFailure};

typedef void (*PEEK_HANDLER)
    (void* parm,bool isReceive, unsigned char byte);
// If registered PEEK_HANDLER is called for every byte transmitted or received


class WatchDog;
class DLList;

class Gpib 
{
public:
    Gpib();
    int registerName(const char *gpibName);
    static Gpib *find(const char *gpibName);
    static bool setSniffHandler(
        const char *linkName,void* phParm,PEEK_HANDLER ph);
    static void freeSniffHandler(const char *linkName);
    // cmd returns actual number bytes transfered
    virtual int cmd(const char *buf, int cnt, int time_limit) = 0;
    // write returns actual number bytes transfered
    virtual int write(const char *buf, int cnt,
	int address, int time_limit) = 0;
    virtual gpibStatus read(char *buf, int cnt, int *actual,
	int address, int time_limit, int eos) = 0;
    gpibStatus getStatus() const {return(ioStatus);}
    const char *getGpibName();
protected:
    void startIoTimeout(int seconds);
    void waitIo();
    void waitIo(int seconds);
    gpibStatus ioStatus;
    PEEK_HANDLER ph;
    void* phParm;
    SEM_ID ioComplete;
private:
    const char *gpibName;
    static void timoutHandler(void *pvt);
    WatchDog *pwg;
};

#endif // GpibH
