// GpibGsTi9914.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/

// Author: John Winans
// Date: 12/94
// Change from Hideos to MPF: Marty Kraimer July 1998

// Support for the IP-488 (Green Springs GPIB IP using the ti9914)

#include <vxWorks.h>
#include <iv.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
#include <intLib.h>

#include "WatchDog.h"
#include "Reboot.h"
#include "DLList.h"
#include "IndustryPackModule.h"
#include "Gpib.h"

class GpibGS : public Gpib
{
public:
    GpibGS(const char *moduleName, IndustryPackModule *pIndustryPackModule,
        int intVec);
    static GpibGS *findGpibGS(const char *moduleName);
    virtual int cmd(const char *buf, int cnt, int time_limit);
    virtual int write(const char *buf, int cnt,
	int address, int time_limit);
    virtual gpibStatus read(char *buf, int cnt, int *actual,
	int address, int time_limit, int eos);
private:
    static void interruptHandler(void*);
    void sicGpib(void);
    int cmdGpib(const char *buf, int cnt, int time_limit);
    int writeGpib(const char *buf, int cnt, bool raw_flag, int time_limit);
    int readGpib(char *buf, int cnt, int time_limit, int eos);
    static void writeIpReg(volatile unsigned short *dest,unsigned short value);
    static unsigned short readIpReg(volatile unsigned short *src);
    static void rebootCallback(void *);
    
    IndustryPackModule *pIPM;		// my module descriptor
    const char *name;
    int intVec;
    volatile struct ip488RegisterMap	*regs;	// hardware registers
    unsigned short copyIsr0; // can be altered in the IRQ handler
    unsigned short CopyIsr1; // can be altered in the IRQ handler
    bool	readHoldof;
    int	transferSize;
    unsigned char	*transferBuffer;
    bool	transferEoiOnLast;
    enum	{IP488_READ, IP488_WRITE, IP488_IDLE} transferState;
    int   transferEos;

};
 
// following necessary to delay ip register  accesses; OK for mv172
#define IP_DELAY_LOOPS 100

int GpibGsTi9914Debug=0;

typedef struct  ip488RegisterMap
{
  unsigned short intReg0;
  unsigned short intReg1;
  unsigned short addressStatus;
  unsigned short cmdStatus;
  unsigned short address;
  unsigned short serialPoll;
  unsigned short commandPasss;
  unsigned short data;
  unsigned short addressSwitch;
  unsigned short vectorRegister;
} ip488RegisterMap;

#define GREEN_SPRING_ID  0xf0
#define GSIP_488  0x14

// Hardware register bit definitions 
//      Name            Bit(s)       register
#define isr0END         (1<<3)		// EOI present on bus
#define isr0BO          (1<<4)		// Ready to write a byte
#define isr0BI          (1<<5)		// Ready to read byte

#define isr1SRQ         (1<<1)		// SRQ on bus is active
#define isr1ERR		(1<<6)		// handshake error (no listener)

#define IP488_CIC_ADDR  0       // Primary address of the TI9914 controller
 
/* TI9914 Auxiliary Commands */
#define IP488_AUXswrstC    0x00    /* Software reset CLEAR              */
#define IP488_AUXswrstS    0x80    /* Software reset SET                */
#define IP488_AUXdacrC     0x01    /* Release DAC holdoff CLEAR         */
#define IP488_AUXdacrS     0x81    /* Release DAC holdoff SET           */
#define IP488_AUXrhdf      0x02    /* Release RFD holdoff               */
#define IP488_AUXhdfaC     0x03    /* Holdoff on all data CLEAR         */
#define IP488_AUXhdfaS     0x83    /* Holdoff on all data SET           */
#define IP488_AUXhdfeC     0x04    /* Holdoff on EOI only CLEAR         */
#define IP488_AUXhdfeS     0x84    /* Holdoff on EOI only SET           */
#define IP488_AUXnbaf      0x05    /* New byte available false          */
#define IP488_AUXfgetC     0x06    /* Force group execute trigger CLEAR */
#define IP488_AUXfgetS     0x86    /* Force group execute trigger SET   */
#define IP488_AUXrtlC      0x07    /* Return to local CLEAR             */
#define IP488_AUXrtlS      0x87    /* Return to local SET               */
#define IP488_AUXfeoi      0x08    /* Send EOI with next byte           */
#define IP488_AUXlonC      0x09    /* Listen only CLEAR                 */
#define IP488_AUXlonS      0x89    /* Listen only SET                   */
#define IP488_AUXtonC      0x0A    /* Talk only CLEAR                   */
#define IP488_AUXtonS      0x8A    /* Talk only SET                     */
#define IP488_AUXgts       0x0B    /* Go to standby                     */
#define IP488_AUXtca       0x0C    /* Take control asynchronously       */
#define IP488_AUXtcs       0x0D    /* Take control synchronously        */
#define IP488_AUXrppC      0x0E    /* Request parallel poll CLEAR       */
#define IP488_AUXrppS      0x8E    /* Request parallel poll SET         */
#define IP488_AUXsicC      0x0F    /* Send interface clear CLEAR        */
#define IP488_AUXsicS      0x8F    /* Send interface clear SET          */
#define IP488_AUXsreC      0x10    /* Send remote enable CLEAR          */
#define IP488_AUXsreS      0x90    /* Send remote enable SET            */
#define IP488_AUXrqc       0x11    /* Request Control                   */
#define IP488_AUXrlc       0x12    /* Release Control                   */
#define IP488_AUXdaiC      0x13    /* Disable all interrupts CLEAR      */
#define IP488_AUXdaiS      0x93    /* Disable all interrupts SET        */
#define IP488_AUXpts       0x14    /* Pass through next secondary       */
#define IP488_AUXstdlC     0x15    /* Short TI settling time CLEAR      */
#define IP488_AUXstdlS     0x95    /* Short TI settling time SET        */
#define IP488_AUXshdwC     0x16    /* Shadow handshake CLEAR            */
#define IP488_AUXshdwS     0x96    /* Shadow handshake SET              */
#define IP488_AUXvstdlC    0x17    /* Very short T1 delay CLEAR         */
#define IP488_AUXvstdlS    0x97    /* Very short T1 delay SET           */
#define IP488_AUXrsv2C     0x18    /* Request Service Bit 2 CLEAR       */
#define IP488_AUXrsv2S     0x98    /* Request Service Bit 2 SET         */

int initGpibGsTi9914(const char *moduleName, const char *carrierName,
    const char *siteName, int intVec)
{
    if(Gpib::find(moduleName)) {
	printf("initGpibGsTi9914 %s already exists\n",moduleName);
	return(0);
    }
    IndustryPackModule *pIPM = IndustryPackModule::createIndustryPackModule(
	moduleName,carrierName,siteName);
    if(!pIPM) return(0);
    unsigned char manufacturer = pIPM->getManufacturer();
    unsigned char model = pIPM->getModel();
    if(manufacturer!=GREEN_SPRING_ID) {
	printf("initGpibGsTi9914 manufacturer 0x%x not GREEN_SPRING_ID\n",
	    manufacturer);
	return(0);
    }
    if(model!=GSIP_488) {
       printf("initGpibGsTi9914 model 0x%x not a IP-488\n",model);
       return(0);
    }
    new GpibGS(moduleName,pIPM,intVec);
    return(0);
}

GpibGS::GpibGS(const char *name, IndustryPackModule *pIndustryPackModule,
      int intVecIn)
: pIPM(pIndustryPackModule), intVec(intVecIn), readHoldof(false),
  transferSize(0), transferBuffer(0),
  transferEoiOnLast(false), transferState(IP488_IDLE), transferEos(0)
{
    registerName(name);
    if((intVec&1)==0) {
        printf("%s intVec MUST be even. It is changed from %d to %d\n",
            name, intVec, intVec|1);
        intVec |= 1;
    }
    // Find the register map of the IP card
    regs=(ip488RegisterMap *)pIPM->getMemBaseIO();
    if(GpibGsTi9914Debug)printf("GpibGS regs %p\n",regs);
    // configure the interrupts
    pIPM->intConfig(0);
    if(intConnect(INUM_TO_IVEC(intVec),(VOIDFUNCPTR)interruptHandler,(int)this)){
	printf("GpibGsTi9914 %s intConnect failed\n",getGpibName());
    }
    writeIpReg(&regs->vectorRegister,(unsigned short)intVec);
    // Issue a software reset to the GPIB controller chip
    writeIpReg(&regs->cmdStatus,IP488_AUXswrstS);	// assert swrst
    // Set the ti9914's address to use while it is CIC
    writeIpReg(&regs->address,IP488_CIC_ADDR);
    // Toss interrupt status values before we release the reset
    copyIsr0 = readIpReg(&regs->intReg0);
    CopyIsr1 = readIpReg(&regs->intReg1);
    writeIpReg(&regs->cmdStatus,IP488_AUXswrstC);	// CLEAR swrst
    writeIpReg(&regs->intReg0,isr0BO|isr0BI);
    writeIpReg(&regs->intReg1,isr1ERR);
    sicGpib();				// assert IFC
    writeIpReg(&regs->cmdStatus,IP488_AUXsreS);	// assert REN
    writeIpReg(&regs->cmdStatus,IP488_AUXdaiC);	// Clear dai
    // Prime our copies of these registers
    copyIsr0 = readIpReg(&regs->intReg0);   
    CopyIsr1 = readIpReg(&regs->intReg1);
    copyIsr0 = copyIsr0 | isr0BO;
    pIPM->intClear(0);
    pIPM->intEnable(0);
    Reboot::rebootHookAdd(rebootCallback,(void *)this);
}

int GpibGS::cmd(const char *buf, int cnt, int time_limit)
{
  if(GpibGsTi9914Debug) {
    printf("GpibGS::cmd [");
    for (int i=0; i<cnt; i++) printf("%3.2x",buf[i]);
    printf("]\n");
  }
  int status = cmdGpib(buf,cnt,time_limit);
  if(GpibGsTi9914Debug) printf("GpibGS::cmd return(%d)\n",status);
  return(status);
}

int GpibGS::write(const char *buf, int cnt, int addr, int time_limit)
{
    if(GpibGsTi9914Debug) {
        printf("GpibGS::write %d [", addr);
        for (int i=0; i<cnt; i++) printf("%3.2x",buf[i]);
        printf("]\n");
    }
    if(cmdGpib("_?", 2, time_limit) != 2) {
        if(GpibGsTi9914Debug) printf("GpibGS::write cmdGpib error before\n");
	return(-1);
    }
    char address = (char)addr + 0x20;
    if(cmdGpib(&address, 1, time_limit)!=1) {
        if(GpibGsTi9914Debug) printf("GpibGS::write cmdGpib error address\n");
        return(-1);
    }
    int actual = writeGpib(buf,cnt,0,time_limit);
    if(actual!=cnt) return(-1);
    if(ioStatus!=gpibSuccess) return(-1);
    if(cmdGpib("_?", 2, time_limit) != 2) {
        if(GpibGsTi9914Debug) printf("GpibGS::write cmdGpib error after\n");
        return(-1);
    }
    return(actual);
}

gpibStatus GpibGS::read(char *buf, int cnt, int *actual,
    int addr, int time_limit, int eos)
{
    *actual=0; *buf=0;
    if(cmdGpib("_?", 2, time_limit) != 2) {
        if(GpibGsTi9914Debug) printf("GpibGS::read cmdGpib error before\n");
	return(gpibTimeout);
    }
    char address = (char)addr + 0x40;
    if(cmdGpib(&address, 1, time_limit)!=1) {
        if(GpibGsTi9914Debug) printf("GpibGS::read cmdGpib error address\n");
        return(gpibTimeout);
    }
    *actual = readGpib(buf,cnt,time_limit,eos);
    if(cmdGpib("_?", 2, time_limit) != 2) {
        if(GpibGsTi9914Debug) printf("GpibGS::read cmdGpib error after\n");
        return(gpibTimeout);
    }
    if(GpibGsTi9914Debug){
        printf("GpibGS::read %d [", addr);
        for (int i=0; i< *actual; i++) printf("%3.2x",buf[i]);
        printf("]\n");
    }
    return(gpibSuccess);
}

void GpibGS::interruptHandler(void* v)
{
    GpibGS *pGpib = (GpibGS *)v;
    int message_complete = 0;
    pGpib->pIPM->intClear(0);
    pGpib->copyIsr0 |= readIpReg(&pGpib->regs->intReg0);
    pGpib->CopyIsr1 |= readIpReg(&pGpib->regs->intReg1);
    switch (pGpib->transferState)
    {
    case IP488_READ:
        if (pGpib->copyIsr0 & isr0BI)
        { // We are ready to read a byte now
            unsigned char newchar = 0;
            if (pGpib->transferSize == 0)
            { // We are out of buffer space
                newchar = readIpReg(&pGpib->regs->data);
                pGpib->transferState = IP488_IDLE;
                pGpib->ioStatus = gpibFailure;	// buffer overflow
		semGive(pGpib->ioComplete);
                return ;
            }
            else if (pGpib->transferSize == 1)
            { // This is the last character which will fit in
                // the buffer, set hold off
                writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXhdfaS);// Holdoff 
                newchar = readIpReg(&pGpib->regs->data);
                writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXhdfaC);
                *pGpib->transferBuffer = newchar;
                message_complete = 1;
            }
            else if (pGpib->transferEos != -1 ) 
            { // Check for EOS character - must read with holdoff or we
                // will lose the character after the EOS
                writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXhdfaS); // Holdoff
                newchar = readIpReg(&pGpib->regs->data);
                writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXhdfaC);
                *pGpib->transferBuffer = newchar;
                if (*pGpib->transferBuffer == pGpib->transferEos) {
                    // This is EOS character, message complete
                    message_complete = 1;
                } else  {
                    // Not EOS character, release read holdoff
                    writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXrhdf);
                }
            } else {
                newchar = readIpReg(&pGpib->regs->data);
                *pGpib->transferBuffer = newchar;
            }
            if(pGpib->ph) (*pGpib->ph)(pGpib->phParm,true,newchar);

            --(pGpib->transferSize);
            ++(pGpib->transferBuffer);
            pGpib->copyIsr0 &= ~isr0BI;
        }
        if (pGpib->copyIsr0 & isr0END)
        { // EOI was asserted for this character
            pGpib->copyIsr0 &= ~isr0END;
            message_complete = 1;
        }
        if (message_complete)
        {
            pGpib->transferState = IP488_IDLE;
	    pGpib->ioStatus = gpibSuccess;
	    semGive(pGpib->ioComplete);
            return ;
        }
        break;
    case IP488_WRITE:
        // If data transmitter is ready, send next byte
        if (pGpib->CopyIsr1 & isr1ERR)
        { // Handshake failure -- we ain't got a listener
            pGpib->CopyIsr1 &= ~isr1ERR;		// Clear error status
            writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXnbaf);// Loose last byte
            ++(pGpib->transferSize);			// Clean up
            --(pGpib->transferBuffer);			// Clean up
            logMsg("GpibGsTi9914::interruptHandler Handshake failure\n");
            pGpib->transferState = IP488_IDLE;
	    pGpib->ioStatus = gpibFailure;
	    semGive(pGpib->ioComplete);
            return ;
        }
        if (pGpib->copyIsr0 & isr0BO)
        { // We are ready to send a byte now
            unsigned short newchar;
            if (pGpib->transferSize == 0)
            { // Send message to indicate transfer completion
                pGpib->transferState = IP488_IDLE;
	        pGpib->ioStatus = gpibSuccess;
	        semGive(pGpib->ioComplete);
                return ;
            }

            if(pGpib->ph)
                (*pGpib->ph)(pGpib->phParm,false,*pGpib->transferBuffer);
            if ((pGpib->transferSize == 1) && pGpib->transferEoiOnLast) {
                writeIpReg(&pGpib->regs->cmdStatus,IP488_AUXfeoi);
            }
            
            newchar = (unsigned short)(unsigned char)(*pGpib->transferBuffer);
            writeIpReg(&pGpib->regs->data,newchar);
            --(pGpib->transferSize);
            ++(pGpib->transferBuffer);

            pGpib->copyIsr0 &= ~isr0BO;
        }
        break;
    case IP488_IDLE:
        logMsg("GpibGsTi9914::interruptHandler IP488_IDLE; why???\n");
        break;
    }

    return ;
}

void GpibGS::sicGpib(void)
{
    if(GpibGsTi9914Debug) printf("GpibGS::sicGpib\n");
    transferState = IP488_IDLE;
    writeIpReg(&regs->cmdStatus,IP488_AUXsicS);	// assert the IFC line
    taskDelay(1);
    writeIpReg(&regs->cmdStatus,IP488_AUXsicC);	// release the IFC line
}

// Write a string with the ATN line active
int GpibGS::cmdGpib(const char *buf, int cnt, int time_limit)
{
    if (cnt == 0) {
        printf("GpibGS::cmdGpib called with cnt=0\n");
        ioStatus = gpibSuccess;
        return (0);
    }
    // Send the first byte manually
    int lockKey = ::intLock();
    writeIpReg(&regs->cmdStatus,IP488_AUXtca);	// Take control of GPIB
    ioStatus = gpibSuccess;
    transferEoiOnLast = false;
    transferSize = cnt;
    transferBuffer = (unsigned char *)buf;
    transferState = IP488_WRITE;
    interruptHandler(this);
    ::intUnlock(lockKey);
    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    return(cnt - transferSize);
}

int GpibGS::readGpib(char *buf, int cnt, int time_limit, int eos)
{
    if (cnt == 0) {
        printf("GpibGS::readGpib called with cnt=0\n");
        ioStatus = gpibSuccess;
        return (0);
    }
    int lockKey = ::intLock();
    writeIpReg(&regs->cmdStatus,IP488_AUXgts);   // If CAC, go into standby
    writeIpReg(&regs->cmdStatus,IP488_AUXlonS);// force into listener state
    // If are currently holding off on the receiver, finish it
    if(readHoldof) writeIpReg(&regs->cmdStatus,IP488_AUXrhdf);
    readHoldof = true;
    writeIpReg(&regs->cmdStatus,IP488_AUXhdfeS);	// Holdoff on EOI
    ioStatus = gpibSuccess;
    transferSize = cnt;
    transferBuffer = (unsigned char *)buf;
    transferState = IP488_READ;
    transferEos = eos;
    ::intUnlock(lockKey);
    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    writeIpReg(&regs->cmdStatus,IP488_AUXlonC);	// force outta listener state
    return(cnt - transferSize);
}

int GpibGS::writeGpib(const char *buf, int cnt, bool raw_flag, int time_limit)
{
    if (cnt == 0) {
        printf("GpibGS::writeGpib called with cnt=0\n");
        ioStatus = gpibSuccess;
        return (0);
    }
    int lockKey = ::intLock();
    ioStatus = gpibSuccess;
    writeIpReg(&regs->cmdStatus,IP488_AUXgts);	// If CAC, go into standby
    writeIpReg(&regs->cmdStatus,IP488_AUXtonS);	// Force talker on
    if(raw_flag) 
	transferEoiOnLast = false;
    else
	transferEoiOnLast = true;
    transferSize = cnt;
    transferBuffer = (unsigned char *)buf;
    transferState = IP488_WRITE;
    interruptHandler(this);
    ::intUnlock(lockKey);
    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    writeIpReg(&regs->cmdStatus,IP488_AUXtonC);
    return(cnt - transferSize);

}

//writeGpib and readGpib are necessary because a fast processor can access
//ip registers  faster than the TMS9914A chip

void GpibGS::writeIpReg(volatile unsigned short *dest,unsigned short value)
{
    volatile int counter=0;
    for(int i=0; i<IP_DELAY_LOOPS; i++) counter++;
    *dest = value;
}

unsigned short GpibGS::readIpReg(volatile unsigned short *src)
{
    volatile int counter=0;
    for(int i=0; i<IP_DELAY_LOOPS; i++) counter++;
    return(*src);
}

void GpibGS::rebootCallback(void *v)
{
    GpibGS *pGpibGS = (GpibGS *)v;
    pGpibGS->pIPM->intDisable(0);
}
