//SerialPort.h

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
	Current Author: Marty Kraimer, Thomas Coleman
	Original Author: Jim Kowalkowski (for Hideos)
	Date: 09JUL98
*/
 
#ifndef SerialPortH
#define SerialPortH

#include <semLib.h>

enum requestType {requestNone,requestRead,requestWrite,requestWriteRead};
enum byteHandlerRC {byteHandlerOK,byteHandlerEndRead,byteHandlerError};
enum serialPortStatus {serialPortSuccess, serialPortTimeout, serialPortFailure};

typedef byteHandlerRC (*BYTE_HANDLER)
    (void* parm,unsigned char byte);
/*
    byte handler is called by interrupt routine. It must
    byteHandlerOK  normal return
    byteHandlerEndRead end of input message has been detected
    byteHandlerError - An error occured - end operation

*/

typedef void (*PEEK_HANDLER)
    (void* parm,bool isReceive, unsigned char byte);
// If registered PEEK_HANDLER is called for every byte transmitted or received

class WatchDog;
class SerialPort
{
public:
    SerialPort();
    int registerName(const char *portName);
    static SerialPort *find(const char *portName);
    static SerialPort *bind(const char *portName,void* bhParm,BYTE_HANDLER bh);
    void release();
    static bool setSniffHandler(
	const char *portName,void* phParm,PEEK_HANDLER ph);
    static void freeSniffHandler(const char *portName);
    serialPortStatus getStatus() { return(ioStatus);}
    virtual serialPortStatus write(
	unsigned char *data, int nbytes, int timeout) = 0;
    virtual serialPortStatus read(int timeout) =0;
    virtual serialPortStatus writeRead(
	unsigned char *data, int nbytes, int timeout) =0;
    virtual bool config( int baud, int stopBits, int bitsPerChar,
	char parity, char flowControl) = 0;
protected:
    void startIoTimeout(int seconds);
    void waitIo();
    requestType request;
    serialPortStatus ioStatus;
    BYTE_HANDLER bh;
    void* bhParm;
    PEEK_HANDLER ph;
    void* phParm;
    SEM_ID ioComplete;
private:
    static void timoutHandler(void *pvt);
    WatchDog *pwg;
    bool isBound;
    const char *portName;
    // disallow
};

#endif // SerialPortH
