// Float64ArrayMessage.cc

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "DLList.h"
#include "DataFreeList.h"
#include "Float64ArrayMessage.h"

DLList* Float64ArrayMessage::pFreeList = 0;

int32 Float64ArrayMessage:: toBuffer(char **buffer) const
{
    int32 nbytes;

    nbytes = StandardFieldsMessage::toBuffer(buffer);
    nbytes += MPF::toBuffer(buffer,(int32)(size/sizeof(float64)));
    nbytes += MPF::toBuffer(buffer,(const void *)value,size);
    return(nbytes);
}

int32 Float64ArrayMessage:: fromBuffer(const char **buffer)
{
    int32 nbytes=0;
    int32 length=0;
    
    nbytes = StandardFieldsMessage::fromBuffer(buffer);
    nbytes += MPF::fromBuffer(buffer,&length);
    size = length*sizeof(float64);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Float64ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    nbytes += MPF::fromBuffer(buffer,(void *)value,size);
    return(nbytes);
}

int32 Float64ArrayMessage:: fromBufferSwitch(const char **buffer)
{
    int32 nbytes=0;
    int32 length=0;

    nbytes = StandardFieldsMessage::fromBufferSwitch(buffer);
    nbytes += MPF::fromBufferSwitch(buffer,&length);
    size = length*sizeof(float64);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Float64ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    for(int i=0; i<length; i++) 
      nbytes += MPF::fromBufferSwitch(buffer,&value[i]);

    return(nbytes);
}

void Float64ArrayMessage:: print() const
{
    int32 length = size/sizeof(float64);

    StandardFieldsMessage::print();
    printf("maxLength %d length %d value %p",
	   (int32)(maxSize/sizeof(float64)),length,value);
    for(int i=0; i<length; i++) printf(" %f",value[i]);
    printf("\n");
}

void *Float64ArrayMessage:: operator new(size_t size)
{
    if(size==sizeof(Float64ArrayMessage)) return MPF:: allocate(&pFreeList,size);
    printf("Float64ArrayMessage new size !=sizeof(Float64ArrayMessage)\n");
    return(0);
}
 
void Float64ArrayMessage:: operator delete(void *p, size_t size)
{
    Float64ArrayMessage *pmessage = (Float64ArrayMessage *)p;
    if(size==sizeof(Float64ArrayMessage)) {
	pmessage->freeValue();
	MPF::free(pFreeList,p,size);
	return;
    }
    printf("Float64ArrayMessage delete size !=sizeof(Float64ArrayMessage)\n");
}

void Float64ArrayMessage::setLength(int32 length)
{
    if((int32)(length*sizeof(float64))>maxSize) {
	printf("Float64ArrayMessage::setLength length %d > maxLength %d\n",
	    length,(int32)(maxSize*sizeof(float64)));
	this->size = maxSize;
    } else 
	this->size = length*sizeof(float64);
}

void Float64ArrayMessage::allocValue(int32 maxLength)
{
    DataFreeList::alloc(maxLength*sizeof(float64),(void **)&value,&this->maxSize);
}

void Float64ArrayMessage::freeValue()
{
    if(maxSize==0) return;
    DataFreeList::free((void *)value,maxSize);
    maxSize=0; value=0;
}
